package com.ximba.videolib;

import java.io.*;
import java.util.*;
import java.net.*;
import org.apache.log4j.Logger;

/* Project Imports */
import com.ximba.common.*;

/**
 * <p> 
 * A Video entry.
 * </p>
 * @author Michael J. Hammel
 */

public class Video implements Serializable {

    /** lo4j logger */
    private static final Logger log = Logger.getLogger("com.ximba.videolib.Video");

    // A place to store configuration settings.
    private static final List<Video> db = new ArrayList<Video>();

    // Fields from tmdb.org
    private String path = null;
    private int    tmdbID = -1;
    private String poster = null;
    private String releaseDate = null;
    private String overview = null;

    // Used when scanning a directory
    private File   file;        // The file generated by the scan
    private String displayName; // The name displayed in the UI's list and used to query the db.

    /*
     * =======================================================
     * Constructor
     * =======================================================
     */

    public Video() {}

    /*
     * =======================================================
     * Public methods 
     * =======================================================
     */

    /*
     * =======================================================
     * Getters
     * =======================================================
     */
    static public String hdrs() {
        StringBuilder str = new StringBuilder("");
        str.append( "Path;" );
        str.append( "DisplayName;" );
        str.append( "TMDB ID;" );
        str.append( "Poster;" );
        str.append( "Release Date;" );
        str.append( "Overview" );
        return str.toString();
    }

    public File   getFile()        { return file; }
    public String getPath()        { return this.path; }
    public String getDisplayName() { return this.displayName; }
    public int    getID()          { return this.tmdbID; }
    public String getPoster()      { return this.poster; }
    public String getReleaseDate() { return this.releaseDate; }
    public String getOverview()    { return this.overview; }

    // Dirname is used to strip of absolute path
    public String toString( String dirName ) 
    { 
        StringBuilder str = new StringBuilder("");

        if ( getPath() != null ) str.append( getPath().replace(dirName,"") + ";" );
        else                   str.append( " ;" );

        if ( getDisplayName() != null ) str.append( getDisplayName() + ";" );
        else                            str.append( " ;" );

        str.append( ""+getID() + ";" );

        if ( getPoster() != null ) str.append( getPoster() + ";" );
        else                       str.append( " ;" );

        if ( getReleaseDate() != null ) str.append( getReleaseDate() + ";" );
        else                            str.append( " ;" );

        if ( getOverview() != null ) str.append( getOverview() );
        else                         str.append( getOverview() );

        return str.toString();
    }

    /*
     * =======================================================
     * Setters
     * =======================================================
     */

    public void setPath( String path ) { this.path = path; }
    public void setDisplayName( String displayName ) { this.displayName = displayName; }

    // Data from TMDB.org
    public void setID( int id ) { this.tmdbID = id; }
    public void setPoster( String poster ) { this.poster = poster; }
    public void setReleaseDate( String releaseDate ) { this.releaseDate = releaseDate; }
    public void setOverview( String overview ) { this.overview = overview; }

    // Called when scanning a directory 
    public void setFile( File file, String strip )
    {
        this.file = file;
        if ( strip != null )
            setPath( file.getPath().replace(strip,"") );
        else
            setPath( file.getPath() );

        // displayName is what is displayed in the Movie List in the UI.
        this.displayName = file.getName().replace("_", " ");
        for(int i=0; i<UI.extensions.length; i++)
            this.displayName = this.displayName.replace(UI.extensions[i], "");
        log.trace("Set displayName: " + this.displayName);
    }
}
